@@ -25,6 +25,6 @@ system.add_command(grp_central_core.central_core)
 system.add_command(grp_roles.roles)
 system.add_command(grp_settings.settings_lifecycle)
 system.add_command(grp_settings.settings_gui)
-system.add_command(grp_settings.settings_core)
+system.add_command(grp_settings.settings_global)
 system.add_command(grp_users.users)
 system.add_command(grp_discover.discover)
