@@ -1,41 +1,41 @@
 # -*- coding: utf-8 -*-
 """Version information for this package."""
-__version__ = "4.0.1"
+__version__ = "4.1.0"
 VERSION: str = __version__
-""":obj:`str`: Version of package."""
+"""Version of package."""
 
 __url__ = "https://github.com/Axonius/axonius_api_client"
 URL: str = __url__
-""":obj:`str`: URL of package."""
+"""URL of package."""
 
 __author__ = "Axonius"
 AUTHOR: str = __author__
-""":obj:`str`: Auth of package."""
+"""Auth of package."""
 
 __title__ = "axonius_api_client"
 TITLE: str = __title__
-""":obj:`str`: Title of package."""
+"""Title of package."""
 
 __project__ = "axonius_api_client"
 PROJECT: str = __project__
-""":obj:`str`: Name of package."""
+"""Name of package."""
 
 __author_email__ = "support@axonius.com"
 AUTHOR_EMAIL: str = __author_email__
-""":obj:`str`: Author email of package."""
+"""Author email of package."""
 
 __description__ = "Axonius API client for Python"
 DESCRIPTION: str = __description__
-""":obj:`str`: Description of package."""
+"""Description of package."""
 
 __docs__ = "https://axonius-api-client.readthedocs.io/en/latest/"
 DOCS: str = __docs__
-""":obj:`str`: Link to the documentation for this package."""
+"""Link to the documentation for this package."""
 
 __license__ = "MIT"
 LICENSE: str = __license__
-""":obj:`str`: License of package."""
+"""License of package."""
 
 __copyright__ = "Copyright Axonius 2020"
 COPYRIGHT: str = __copyright__
-""":obj:`str`: Copyright of package."""
+"""Copyright of package."""
